#!/bin/php
<?php
/**
 * @package JoomlaPack
 * @subpackage Utility
 */

// Do not allow being called as a web script
if( array_key_exists('REQUEST_METHOD', $_SERVER) ) die("<h1>Use the command line, Luke!</h1><p>Which means, you should call this script <b>from a command line</b> and not as a web script! You <i>did</i> read the README. Or didn't you?</p>\n");

define( '_JPA_MAJOR', 1 ); // JPA Format major version number
define( '_JPA_MINOR', 0 ); // JPA Format minor version number

class JPA
{
	/**
	 * Full pathname to the archive file
	 *
	 * @var string
	 */
	var $_dataFileName;

	/**
	 * Number of files in the archive
	 *
	 * @var int
	 */
	var $_fileCount;

	/**
	 * Size in bytes of the data read from the filesystem
	 *
	 * @var int
	 */
	var $_uncompressedSize;

	/**
	 * Size in bytes of the data stored in the archive
	 *
	 * @var int
	 */
	var $_compressedSize;

	/**
	 * Beginning of central directory record.
	 *
	 * @var string
	 */
	var $_ctrlDirHeader = "\x4A\x50\x41";	// Standard Header signature

	/**
	 * Beginning of file contents.
	 *
	 * @var string
	 */
	var $_fileHeader = "\x4A\x50\x46";	// Entity Block signature

	/**
	 * Error message
	 *
	 * @var string
	 */
	var $error;

	function JPA( $archive )
	{
		$this->_dataFileName = $archive;

		// Try to kill the archive if it exists
		$fp = @fopen( $this->_dataFileName, "wb" );
		if (!($fp === false)) {
			@ftruncate( $fp,0 );
			@fclose( $fp );
		} else {
			if( file_exists($this->_dataFileName) ) @unlink( $this->_dataFileName );
			@touch( $this->_dataFileName );
		}

		if(!is_writable($archive))
		{
			$this->error = 'Can\'t open '.$archive.' for writing';
			return;
		}

		// Write the initial instance of the archive header
		$this->_writeArchiveHeader();
	}

	function addFile($from)
	{
		// See if it's a directory
		$isDir = is_dir($from);
		// Get real size before compression
		$fileSize = $isDir ? 0 : filesize($from);
		// Decide if we will compress
		if ($isDir) {
			$compressionMethod = 0; // don't compress directories...
		} else {
			$compressionMethod = 1; // Compress all files, since command line doesn't suffer from timeouts
		}
		$compressionMethod = function_exists("gzcompress") ? $compressionMethod : 0;
		$storedName = $from;

		/* "Entity Description Block" segment. */
		$unc_len = &$fileSize; // File size
		$storedName .= ($isDir) ? "/" : "";
		if ($compressionMethod == 1) {
			if( function_exists("file_get_contents") )
			{
				$udata = @file_get_contents( $from );
			}
			else
			{
				// Argh... the hard way!
				$udatafp = @fopen( $from, "rb" );
				if( !($udatafp === false) ) {
					$udata = "";
					while( !feof($udatafp) ) {
						$udata .= fread($udatafp, 524288);
					}
					fclose( $udatafp );
				} else {
					$udata = false;
				}
			}
			 
			if ($udata === FALSE) {
				return false;
			} else {
				// Proceed with compression
				$zdata   = @gzcompress($udata);
				if ($zdata === false) {
					// If compression fails, let it behave like no compression was available
					$c_len = &$unc_len;
					$compressionMethod = 0;
				} else {
					unset( $udata );
					$zdata   = substr(substr($zdata, 0, strlen($zdata) - 4), 2);
					$c_len   = strlen($zdata);
				}
			}
		}
		else
		{
			$c_len = $unc_len;
		}

		$this->_compressedSize += $c_len; // Update global data
		$this->_uncompressedSize += $fileSize; // Update global data
		$this->_fileCount++;

		// Get file permissions
		$perms = @fileperms( $from );

		// Calculate Entity Description Block length
		$blockLength = 21 + strlen($storedName) ;

		// Open data file for output
		$fp = @fopen( $this->_dataFileName, "ab");
		if ($fp === false)
		{
			$this->error = "Could not open archive file '{$this->_dataFileName}' for append!";
			return;
		}

		$this->_fwrite( $fp, $this->_fileHeader ); // Entity Description Block header
		if($this->error) return;

		$this->_fwrite( $fp, pack('v', $blockLength) ); // Entity Description Block header length
		$this->_fwrite( $fp, pack('v', strlen($storedName) ) ); // Length of entity path
		$this->_fwrite( $fp, $storedName ); // Entity path
		$this->_fwrite( $fp, pack('C', ($isDir ? 0 : 1) ) ); // Entity type
		$this->_fwrite( $fp, pack('C', $compressionMethod ) ); // Compression method
		$this->_fwrite( $fp, pack('V', $c_len ) ); // Compressed size
		$this->_fwrite( $fp, pack('V', $unc_len ) ); // Uncompressed size
		$this->_fwrite( $fp, pack('V', $perms ) ); // Entity permissions

		/* "File data" segment. */
		if ($compressionMethod == 1) {
			// Just dump the compressed data
			$this->_fwrite( $fp, $zdata );
			if($this->error) return;
			unset( $zdata );
		} elseif (!$isDir) {
			// Copy the file contents, ignore directories
			$zdatafp = @fopen( $from, "rb" );
			while( !feof($zdatafp) ) {
				$zdata = fread($zdatafp, 524288);
				$this->_fwrite( $fp, $zdata );
				if($this->error) return;
			}
			fclose( $zdatafp );
		}

		fclose( $fp );

		// ... and return TRUE = success
		return TRUE;
	}

	/**
	 * Updates the Standard Header with current information
	 */
	function finalize()
	{
		$this->_writeArchiveHeader();
		if($this->error) return;
	}

	/**
	 * Outputs a Standard Header at the top of the file
	 *
	 */
	function _writeArchiveHeader()
	{
		$fp = @fopen( $this->_dataFileName, 'r+' );
		if($fp === false)
		{
			$this->error = 'Could not open '.$this->_dataFileName.' for writing. Check permissions and open_basedir restrictions.';
			return;
		}
		$this->_fwrite( $fp, $this->_ctrlDirHeader );					// ID string (JPA)
		if($this->error) return;
		$this->_fwrite( $fp, pack('v', 19) );							// Header length; fixed to 19 bytes
		$this->_fwrite( $fp, pack('C', _JPA_MAJOR ) );					// Major version
		$this->_fwrite( $fp, pack('C', _JPA_MINOR ) );					// Minor version
		$this->_fwrite( $fp, pack('V', $this->_fileCount ) );			// File count
		$this->_fwrite( $fp, pack('V', $this->_uncompressedSize ) );	// Size of files when extracted
		$this->_fwrite( $fp, pack('V', $this->_compressedSize ) );		// Size of files when stored
		@fclose( $fp );
	}

	/**
	 * Write to file, defeating magic_quotes_runtime settings (pure binary write)
	 * @param handle $fp Handle to a file
	 * @param string $data The data to write to the file
	 * @access protected
	 */
	function _fwrite( $fp, $data )
	{
		$len = strlen( $data );
		$ret = fwrite( $fp, $data, $len );
		if($ret === FALSE)
		{
			$this->error = "Can't write to archive";
			return false;
		}

		return true;
	}

}

function getDirContents( $dirName, $shellFilter = null )
{
	$handle = @opendir( $dirName );

	// If directory is not accessible, just return FALSE
	if ($handle === FALSE) {
		$false = false;
		return $false;
	}

	// Initialize return array
	$retArray = array();

	// FIX 1.2.1 -- Remove trailing slash
	if( (substr($dirName,-1,1) == '/') || (substr($dirName,-1,1) == '\\')) $dirName = substr($dirName,0,strlen($dirName)-1);

	while( !( ( $filename = readdir($handle) ) === false) ) {
		$match = is_null( $shellFilter );
		$match = (!$match) ? fnmatch($shellFilter, $filename) : true;
		if ($match) {
			$filename = TranslateWinPath( $dirName . DIRECTORY_SEPARATOR . $filename );
			if( (substr($filename, -2) == '/.') || (substr($filename, -3) == '/..' ) )
			{
				// nada
			}
			else
			{
				$retArray[] = $filename;
			}
		}
	}

	@closedir($handle);
	return $retArray;
}

function addRecursive( &$jpa, $filepath )
{
	if(defined('NOSVN'))
	{
		if(substr($filepath, -5) == '/.svn') return;
	}

	if( is_dir($filepath) )
	{
		$fileList = getDirContents($filepath);
		if($fileList !== false)
		{
			foreach($fileList as $file)
			{
				addRecursive($jpa, $file);
			}
		}
		else
		{
			echo $filepath;
			$ret = $jpa->addFile($filepath);
			$pad = 80 - strlen($filepath) - 6;
			if($pad > 0) echo str_repeat(" ", $pad);
			echo $ret ? "[ OK ]" : "[FAIL]";
		}
	}
	else
	{
		echo $filepath;
		$ret = $jpa->addFile($filepath);
		$pad = 80 - strlen($filepath) - 6;
		if($pad > 0) echo str_repeat(" ", $pad);
		echo $ret ? "[ OK ]" : "[FAIL]";
	}
}

/**
 * Makes a Windows path more UNIX-like, by turning backslashes to forward slashes
 *
 * @param string $p_path The path to transform
 * @return string
 */
function TranslateWinPath( $p_path )
{
	if (stristr(php_uname(), 'windows')){
		// Change potential windows directory separator
		if ((strpos($p_path, '\\') > 0) || (substr($p_path, 0, 1) == '\\')){
			$p_path = strtr($p_path, '\\', '/');
		}
	}
	return $p_path;
}

function showHelp()
{
	echo <<<ENDOFHELP
JoomlaPack JPA Archiver

Usage: /path/to/php [--nosvn] jpa.php archive.jpa <files>

Where:
    /path/to/jpa    The full path to the php executable (php.exe under Windows)
    jpa.php         The name of this file
    --nosvn			If specified, it'll skip .svn subdirectories
    archive.jpa     The name of the JPA archive you'd like to extract
    <files>         A list of files and directories to add. Directories are
                    added recursively.

Existing archives are deleted and created anew. You have been warned!
ENDOFHELP;
}

// =============================================================================
// Main loop
// =============================================================================

// Die if insufficient options specified
if( count($argv) < 2 )
{
	showHelp();
	exit;
}

$parameters = $argv;

$archiveName = array_shift($parameters);
// When called using PHP, the first argv contains ourselves. If called directly,
// using the shabang sequence on the top of the file to force UNIX systems to
// parse it using PHP, the first argv contains the archive, I think.
if(strpos($archiveName, basename(__FILE__)) !== false)
{
	$archiveName = array_shift($parameters);
}

// Die if no options specified
if( count($parameters) == 0 )
{
	showHelp();
	exit;
}

// Check for --nosvn
if($archiveName == "--nosvn")
{
	$archiveName = array_shift($parameters);
	define('NOSVN', 1);
	echo "*** SVN directories will be skipped ***\n";
}

// Die if no options specified
if( count($parameters) == 0 )
{
	showHelp();
	exit;
}

$archiveName = TranslateWinPath($archiveName);

echo "New JPA archive $archiveName\n";
$archiver = new JPA($archiveName);
if($archiver->error)
{
	echo $archiver->error . "\n";
	exit(1);
}

foreach($parameters as $newDir)
{
	$newDir = TranslateWinPath($newDir);

	// Detect patterns (parameters containing * and ? in their last part)
	if( (strpos($newDir, '*') !== false) || (strpos($newDir, '?') !== false) )
	{
		$lastslash = strrpos($newDir, '/');
		if($lastslash === false)
		{
			$dir = '.';
			$pattern = $newDir;
		}
		else
		{
			$dir = substr($newDir, 0, $lastslash);
			$pattern = substr($newDir, $lastslash+1, strlen($newDir) - $lastslash );
		}
	}
	else
	{
		$dir = $newDir;
		$pattern = null;
	}

	if(is_dir($dir))
	{
		$fileList = getDirContents($dir, $pattern);
		if($fileList !== false)
		{
			foreach($fileList as $filepath)
			{
				addRecursive($archiver, $filepath);
			}
		}
		else
		{
			// Empty dir
			addRecursive($archiver, $dir);
		}
	}
	else
	{
		addRecursive($archiver, $dir);
	}


}

$archiver->finalize();

$ratio = round(100 * $archiver->_compressedSize / $archiver->_uncompressedSize);
echo "\n\n";
echo "Total files: ".$archiver->_fileCount."\n";
echo "Compression ratio: $ratio%\n";
?>