<?php
/**
 * @package JoomlaPack
 * @subpackage Remote
 * @copyright Copyright (c)2006-2008 JoomlaPack Developers
 * @license GNU General Public License version 2, or later
 * @version $id$
 * @since 2.0
 *
 * Joomla! XMLRPC services plugin for interfacing with JoomlaPack Remote
 */
// no direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

// Do not change this; this is used by the desktop application to validate
// its compatibility with the server it's talking with.
define('JPRemoteAPIVersion','2.1'); // API 2.1 - Native Tools 2009.4

// Import Joomla! plugin functionality
jimport( 'joomla.plugin.plugin' );

/**
 * JoomlaPack Remoting Services XMLRPC plugin class
 */
class plgXMLRPCJPRemote extends JPlugin
{
	function plgXMLRPCJPRemote(&$subject, $config)
	{
		parent::__construct($subject, $config);
	}

	function onGetWebServices()
	{
		// Get the global XML-RPC types
		global $xmlrpcI4, $xmlrpcInt, $xmlrpcBoolean, $xmlrpcDouble, $xmlrpcString,
		$xmlrpcDateTime, $xmlrpcBase64, $xmlrpcArray, $xmlrpcStruct, $xmlrpcValue;

		// Return the services definitions
		return array(
			'joomlapack.getAPI' => array
		(
				'function' => 'plgXMLRPCJPRemoteServices::getAPI',
				'docstring' => 'Gets the JoomlaPack Remoting Services API version',
				'signature' => array(array($xmlrpcString))
		),
			
			'joomlapack.getProfiles' => array
		(
				'function' => 'plgXMLRPCJPRemoteServices::getProfiles',
				'docstring' => 'Retrieves a list of valid backup profile ID\'s and their descriptions.',
				'signature' => array(array($xmlrpcArray, $xmlrpcString, $xmlrpcString))
		),

			'joomlapack.getBackups' => array
		(
				'function' => 'plgXMLRPCJPRemoteServices::getBackups',
				'docstring' => 'Retrieves a list of all backup attempts.',
				'signature' => array(array($xmlrpcArray, $xmlrpcString, $xmlrpcString))
		),

			'joomlapack.startBackup' => array
		(
				'function' => 'plgXMLRPCJPRemoteServices::startBackup',
				'docstring' => 'Starts a new backup attempt.',
				'signature' => array(
					array($xmlrpcStruct, $xmlrpcString, $xmlrpcString, $xmlrpcInt, $xmlrpcString, $xmlrpcString),
					array($xmlrpcStruct, $xmlrpcString, $xmlrpcString, $xmlrpcInt, $xmlrpcString),
					array($xmlrpcStruct, $xmlrpcString, $xmlrpcString, $xmlrpcInt),
					array($xmlrpcStruct, $xmlrpcString, $xmlrpcString)
				)
		),
			
			'joomlapack.continueBackup' => array
		(
				'function' => 'plgXMLRPCJPRemoteServices::continueBackup',
				'docstring' => 'Continues an already started backup attempt.',
				'signature' => array(
					array($xmlrpcStruct, $xmlrpcString, $xmlrpcString, $xmlrpcInt),
					array($xmlrpcStruct, $xmlrpcString, $xmlrpcString)
				)
		),

			'joomlapack.getFilename' => array
		(
				'function' => 'plgXMLRPCJPRemoteServices::getFilename',
				'docstring' => 'Gets the backup archive filename.',
				'signature' => array(
					array($xmlrpcString, $xmlrpcString, $xmlrpcString, $xmlrpcInt),
					array($xmlrpcString, $xmlrpcString, $xmlrpcString)
				)
		),
			
		/* API Version 2.0 features */
			
			'joomlapack.deleteFile' => array
			(
				'function' => 'plgXMLRPCJPRemoteServices::deleteFile',
				'docstring' => 'Deletes the backup archive file of a given backup record.',
				'signature' => array(
					array($xmlrpcStruct, $xmlrpcString, $xmlrpcString, $xmlrpcInt),
				)
			),
			
		/* API Version 2.1 features */
			'joomlapack.getParts' => array
		(
			'function' => 'plgXMLRPCJPRemoteServices::getParts',
			'docstring' => 'Gets the names of all the files this backup consists of.',
			'signature' => array(
					array($xmlrpcArray, $xmlrpcString, $xmlrpcString, $xmlrpcInt, $xmlrpcInt),
					array($xmlrpcArray, $xmlrpcString, $xmlrpcString, $xmlrpcInt),
					array($xmlrpcArray, $xmlrpcString, $xmlrpcString)
			)
		)
		);
	}
}

/**
 * This class implements the web services part of this plugin
 * @author sledge81
 *
 */
class plgXMLRPCJPRemoteServices
{
	/**
	 * Returns the implemented JoomlaPack Remoting Services API version
	 * @return string;
	 */
	function getAPI()
	{
		return JPRemoteAPIVersion;
	}

	/**
	 * Returns a list of backup profile ID's and descriptions
	 *
	 * @param string $username A valid Joomla! user's username
	 * @param string $password The user's password
	 * @return unknown_type
	 */
	function getProfiles($username, $password)
	{
		// Get the global XML-RPC types
		global $xmlrpcI4, $xmlrpcInt, $xmlrpcBoolean, $xmlrpcDouble, $xmlrpcString,
		$xmlrpcDateTime, $xmlrpcBase64, $xmlrpcArray, $xmlrpcStruct, $xmlrpcValue,
		$xmlrpcerruser;

		// Load JoomlaPack base classes
		plgXMLRPCJPRemoteHelper::JoomlaPack_Bootstrap();

		// Authenticate user
		if(!plgXMLRPCJPRemoteHelper::authenticate($username, $password))
		return new xmlrpcresp(0, $xmlrpcerruser+1, JText::_("Login Failed"));
			
		// Do the job
		jpimport('models.profiles',true);
		$profilesModel = new JoomlapackModelProfiles();
		$profilesList = $profilesModel->getProfilesList(true);

		$structarray = array();
		foreach($profilesList as $profile)
		{
			$aProfile = new xmlrpcval(array(
				'id' => new xmlrpcval($profile->id, $xmlrpcInt),
				'description' => new xmlrpcval($profile->description, $xmlrpcString)
			), 'struct');
			$structarray[] = $aProfile;
		}
		return new xmlrpcresp(new xmlrpcval( $structarray , $xmlrpcArray));
	}

	function getBackups($username, $password)
	{
		// Get the global XML-RPC types
		global $xmlrpcI4, $xmlrpcInt, $xmlrpcBoolean, $xmlrpcDouble, $xmlrpcString,
		$xmlrpcDateTime, $xmlrpcBase64, $xmlrpcArray, $xmlrpcStruct, $xmlrpcValue,
		$xmlrpcerruser;

		// Load JoomlaPack base classes
		plgXMLRPCJPRemoteHelper::JoomlaPack_Bootstrap();

		// Authenticate user
		if(!plgXMLRPCJPRemoteHelper::authenticate($username, $password))
		return new xmlrpcresp(0, $xmlrpcerruser+1, JText::_("Login Failed"));
			
		// Do the job
		jpimport('models.statistics', true);
		$model = new JoomlapackModelStatistics();
		$allEntries =& $model->getStatisticsListWithMeta(true);

		$structarray = array();
		foreach($allEntries as $entry)
		{
			$aProfile = new xmlrpcval(array(
				'id' => new xmlrpcval($entry->id, $xmlrpcInt),
				'description' => new xmlrpcval($entry->description, $xmlrpcString),
				'comment' => new xmlrpcval($entry->comment, $xmlrpcString),
				'backupstart' => new xmlrpcval($entry->backupstart, $xmlrpcString),
				'backupend' => new xmlrpcval($entry->backupend, $xmlrpcString),
				'status' => new xmlrpcval($entry->status, $xmlrpcString),
				'origin' => new xmlrpcval($entry->origin, $xmlrpcString),
				'type' => new xmlrpcval($entry->type, $xmlrpcString),
				'meta' => new xmlrpcval($entry->meta, $xmlrpcString),
				'profile' => new xmlrpcval($entry->profile_id, $xmlrpcInt),
				'archivename' => new xmlrpcval($entry->archivename, $xmlrpcString)
			), 'struct');
			$structarray[] = $aProfile;
		}
		return new xmlrpcresp(new xmlrpcval( $structarray , $xmlrpcArray));
	}

	function startBackup($username, $password, $profileid = 1, $description = null, $comment = null)
	{
		// Get the global XML-RPC types
		global $xmlrpcI4, $xmlrpcInt, $xmlrpcBoolean, $xmlrpcDouble, $xmlrpcString,
		$xmlrpcDateTime, $xmlrpcBase64, $xmlrpcArray, $xmlrpcStruct, $xmlrpcValue,
		$xmlrpcerruser;

		// Load JoomlaPack base classes
		plgXMLRPCJPRemoteHelper::JoomlaPack_Bootstrap();

		// Authenticate user
		if(!plgXMLRPCJPRemoteHelper::authenticate($username, $password))
		return new xmlrpcresp(0, $xmlrpcerruser+1, JText::_("Login Failed"));
			
		// Make sure we have a profile set throughout the XMLRPC session
		$session =& JFactory::getSession();
		$session->set('profile', $profileid, 'joomlapack');

		// Validate backup profile ID
		jpimport('models.profiles',true);
		$profilesModel = new JoomlapackModelProfiles();
		$profilesModel->setId($profileid);
		$profile =& $profilesModel->getProfile();
		if(!is_object($profile))
		return new xmlrpcresp(0, $xmlrpcerruser+2, 'Invalid profile ID');

		// Get defaults for any missing fields
		if(is_null($description)) $description = 'XMLRPC remote backup';
		if(is_null($comment)) $comment = '';

		// Cleanup after any previous backup attempts
		jpimport('core.cube');
		JoomlapackCUBE::reset();

		// Remove stale CUBE objects
		jpimport('core.utility.tables');
		JoomlapackCUBETables::DeleteVar('CUBEObject');

		// Load the CUBE and start the backup
		jpimport('models.registry', true);
		$registry =& JoomlapackModelRegistry::getInstance();
		$registry->reload();
		$cube =& JoomlapackCUBE::getInstance();
		$cube->start($description, $comment);
		$ret = $cube->tick();
		$cube->save();

		return new xmlrpcresp(new xmlrpcval( array(
			'IsFinished'	=> new xmlrpcval( $ret['HasRun'] != 0, $xmlrpcBoolean ),
			'Domain'		=> new xmlrpcval( $ret['Domain'], $xmlrpcString ),
			'Step'			=> new xmlrpcval( $ret['Step'], $xmlrpcString ),
			'Substep'		=> new xmlrpcval( $ret['Substep'], $xmlrpcString ),
			'Error'			=> new xmlrpcval( $ret['Error'], $xmlrpcString ),
			'Warnings'		=> new xmlrpcval( $ret['Warnings'], $xmlrpcString )
		) , $xmlrpcStruct));
	}

	function continueBackup($username, $password, $profileid = 1)
	{
		// Get the global XML-RPC types
		global $xmlrpcI4, $xmlrpcInt, $xmlrpcBoolean, $xmlrpcDouble, $xmlrpcString,
		$xmlrpcDateTime, $xmlrpcBase64, $xmlrpcArray, $xmlrpcStruct, $xmlrpcValue,
		$xmlrpcerruser;

		// Load JoomlaPack base classes
		plgXMLRPCJPRemoteHelper::JoomlaPack_Bootstrap();

		// Authenticate user
		if(!plgXMLRPCJPRemoteHelper::authenticate($username, $password))
		return new xmlrpcresp(0, $xmlrpcerruser+1, JText::_("Login Failed"));

		// Make sure we have a profile set throughout the XMLRPC session
		$session =& JFactory::getSession();
		$session->set('profile', $profileid, 'joomlapack');

		// Validate backup profile ID
		jpimport('models.profiles',true);
		$profilesModel = new JoomlapackModelProfiles();
		$profilesModel->setId($profileid);
		$profile =& $profilesModel->getProfile();
		if(!is_object($profile))
		return new xmlrpcresp(0, $xmlrpcerruser+2, 'Invalid profile ID');

		// Load the CUBE and continue the backup
		jpimport('models.registry', true);
		$registry =& JoomlapackModelRegistry::getInstance();
		$registry->reload();
		jpimport('core.cube');
		$cube =& JoomlapackCUBE::getInstance();
		$ret = $cube->tick();
		$cube->save();

		return new xmlrpcresp(new xmlrpcval( array(
			'IsFinished'	=> new xmlrpcval( $ret['HasRun'] != 0, $xmlrpcBoolean ),
			'Domain'		=> new xmlrpcval( $ret['Domain'], $xmlrpcString ),
			'Step'			=> new xmlrpcval( $ret['Step'], $xmlrpcString ),
			'Substep'		=> new xmlrpcval( $ret['Substep'], $xmlrpcString ),
			'Error'			=> new xmlrpcval( $ret['Error'], $xmlrpcString ),
			'Warnings'		=> new xmlrpcval( $ret['Warnings'], $xmlrpcString )
		) , $xmlrpcStruct));
	}

	function getFilename($username, $password, $profileid)
	{
		// Get the global XML-RPC types
		global $xmlrpcI4, $xmlrpcInt, $xmlrpcBoolean, $xmlrpcDouble, $xmlrpcString,
		$xmlrpcDateTime, $xmlrpcBase64, $xmlrpcArray, $xmlrpcStruct, $xmlrpcValue,
		$xmlrpcerruser;

		// Load JoomlaPack base classes
		plgXMLRPCJPRemoteHelper::JoomlaPack_Bootstrap();

		// Authenticate user
		if(!plgXMLRPCJPRemoteHelper::authenticate($username, $password))
		return new xmlrpcresp(0, $xmlrpcerruser+1, JText::_("Login Failed"));
			
		// Validate backup profile ID
		jpimport('models.profiles',true);
		$profilesModel = new JoomlapackModelProfiles();
		$profilesModel->setId($profileid);
		$profile =& $profilesModel->getProfile();
		if(!is_object($profile))
		return new xmlrpcresp(0, $xmlrpcerruser+2, 'Invalid profile ID');

		// Make sure we have a profile set throughout the XMLRPC session
		$session =& JFactory::getSession();
		$session->set('profile', $profileid, 'joomlapack');

		// Load the CUBE and get the filename
		jpimport('core.cube');
		$cube =& JoomlapackCUBE::getInstance();
		$stats = new JoomlapackModelStatistics($cube->_statID);
		$stats->setId($cube->_statID);
		$statRecord =& $stats->getStatistic();
		$filename = $statRecord->archivename;


		return new xmlrpcresp(new xmlrpcval( $filename, $xmlrpcString ));
	}

	function deleteFile($username, $password, $backupid)
	{
		// Get the global XML-RPC types
		global $xmlrpcI4, $xmlrpcInt, $xmlrpcBoolean, $xmlrpcDouble, $xmlrpcString,
		$xmlrpcDateTime, $xmlrpcBase64, $xmlrpcArray, $xmlrpcStruct, $xmlrpcValue,
		$xmlrpcerruser;

		// Load JoomlaPack base classes
		plgXMLRPCJPRemoteHelper::JoomlaPack_Bootstrap();

		// Authenticate user
		if(!plgXMLRPCJPRemoteHelper::authenticate($username, $password))
		return new xmlrpcresp(0, $xmlrpcerruser+1, JText::_("Login Failed"));
			
		// Validate backup record ID
		jpimport('models.profiles',true);
		$statisticsModel =& JoomlapackModelStatistics::getInstance();
		$statisticsModel->setId($backupid);
		$statistic =& $statisticsModel->getStatistic();
		if(!is_object($statistic))
		return new xmlrpcresp(0, $xmlrpcerruser+2, 'Invalid backup record ID');

		// Try to remove the file
		if(!$statistic->removeFile())
		{
			return new xmlrpcresp(0, $xmlrpcerruser+3, 'Error while deleting file: '.$statisticsModel->getError());
		}

		return new xmlrpcresp(new xmlrpcval( $filename, $xmlrpcString ));
	}
	
	function getParts($username, $password, $profileid = 1, $backupid = null)
	{
		// Get the global XML-RPC types
		global $xmlrpcI4, $xmlrpcInt, $xmlrpcBoolean, $xmlrpcDouble, $xmlrpcString,
		$xmlrpcDateTime, $xmlrpcBase64, $xmlrpcArray, $xmlrpcStruct, $xmlrpcValue,
		$xmlrpcerruser;

		// Load JoomlaPack base classes
		plgXMLRPCJPRemoteHelper::JoomlaPack_Bootstrap();

		// Authenticate user
		if(!plgXMLRPCJPRemoteHelper::authenticate($username, $password))
		return new xmlrpcresp(0, $xmlrpcerruser+1, JText::_("Login Failed"));
			
		// Validate backup profile ID
		jpimport('models.profiles',true);
		$profilesModel = new JoomlapackModelProfiles();
		$profilesModel->setId($profileid);
		$profile =& $profilesModel->getProfile();
		if(!is_object($profile))
		return new xmlrpcresp(0, $xmlrpcerruser+2, 'Invalid profile ID');

		// Make sure we have a profile set throughout the XMLRPC session
		$session =& JFactory::getSession();
		$session->set('profile', $profileid, 'joomlapack');

		// Load the CUBE and get the base filename
		jpimport('core.cube');
		if(is_null($backupid))
		{
			$cube =& JoomlapackCUBE::getInstance();
			$backupid = $cube->_statID;
		}

		$stats = new JoomlapackModelStatistics($backupid);
		$stats->setId($backupid);
		$statRecord =& $stats->getStatistic();
		$filename = $statRecord->archivename;
		$multipart = $statRecord->multipart;
		
		$ret = array(
			new xmlrpcval($filename, $xmlrpcString)
		);
		if($multipart > 1)
		{
			$ret = array();
			$allFilenames = $stats->getAllFilenames($backupid);
			if(count($allFilenames) > 0)
			{
				foreach($allFilenames as $filename)
				{
					$ret[] = new xmlrpcval( basename($filename) , $xmlrpcString);
				}
			}
		}
		
		return new xmlrpcresp(new xmlrpcval( $ret, $xmlrpcArray ));
	}
}

/**
 * This is an internal class of utility functions
 *
 */
class plgXMLRPCJPRemoteHelper
{
	/**
	 * Defines path constants which are not defined in XMLRPC mode and pre-loads the
	 * JoomlaPack base classes.
	 *
	 * @access private
	 */
	function JoomlaPack_Bootstrap()
	{
		// Populate missing defines in XMLRPC mode
		define('JPATH_COMPONENT_ADMINISTRATOR', JPATH_ADMINISTRATOR.DS.'components'.DS.'com_joomlapack' );

		// Awful hack: Get the front-end default language by querying the database
		$systemlang = '';
		$db =& JFactory::getDBO();
		$sql = 'SELECT '.$db->nameQuote('params').' FROM '.$db->nameQuote('#__components').
			' WHERE '.$db->nameQuote('option').' = '.$db->Quote('com_languages');
		$db->setQuery($sql);
		$db->query();
		if($db->getNumRows() > 0)
		{
			$paramstring = $db->loadResult();
			$pairs = explode("\n", $paramstring);
			if(count($pairs) > 0)
			{
				foreach($pairs as $string)
				{
					$temp = explode('=', $string, 2);
					if(count($temp) == 2)
					{
						if($temp[0] == 'site') $systemlang = $temp[1];
					}
				}
			}
		}
		
		// Get the languages to be loaded
		$jlang =& JFactory::getLanguage();

		// Frontend language load order: English, system default
		$jlang->load('com_joomlapack', JPATH_ROOT, 'en-GB', true);
		$jlang->load('com_joomlapack', JPATH_ROOT, $systemlang, true);

		$jlang->load('com_joomlapack', JPATH_ADMINISTRATOR, 'en-GB', true);
		$jlang->load('com_joomlapack', JPATH_ADMINISTRATOR, $systemlang, true);

		// Load JoomlaPack Factory and Registry
		require_once JPATH_COMPONENT_ADMINISTRATOR.DS.'classes'.DS.'factory.php';
		jimport('joomla.filesystem.file');
		jpimport('models.registry', true);
	}

	/**
	 * Checks if a username/password pair is a valid login for this Joomla! installation
	 * and has adequate permissions to use JoomlaPack.
	 *
	 * @param string $username Username
	 * @param string $password Password
	 * @return bool True if it is a valid login AND has enought permissions to use JoomlaPack
	 */
	function authenticate($username, $password)
	{
		// Try to authenticate against secret word
		jpimport('models.registry', true);
		$registry =& JoomlapackModelRegistry::getInstance();
		// Is front-end backup enabled?
		if($registry->get('enableFrontend', false))
		{
			// Get the secret word
			$secret_word = $registry->get('secretWord', '');
			// Only check if the secret word is not an empty string
			if( !empty($secret_word) )
			{
				// If the password matches the secret word then return possitive result and quit
				if($secret_word == $password) return true;
			}
		}

		// ********************************************************************
		// This code segment is executed if and only if we didn't have a secret
		// word match in the code above. It might fail if the site has some
		// extremely badly written user or authentication plug-ins installed.
		// ********************************************************************

		// Forcibly use Joomla! Authentication plugin, even if it's not published
		jimport( 'joomla.user.authentication');
		JLoader::import( 'authentication.joomla', JPATH_ROOT.DS.'plugins', 'plugins' );

		$jauth =& JAuthentication::getInstance();
		$auth = new plgAuthenticationJoomla($jauth, array());
		$response = new JAuthenticationResponse();
		$credentials = array( 'username' => $username, 'password' => $password );
		$options = array();
		$auth->onAuthenticate($credentials, $options, $response);

		// CATCH 1: Return false on invalid authentication
		if( !($response->status === JAUTHENTICATE_STATUS_SUCCESS) )
		return false;

		// Login the user
		$user =& JFactory::getUser($username);
		plgXMLRPCJPRemoteHelper::getUserAid($user);

		// Compare user's group to JoomlaPack's minimum privileges setting
		$registry =& JoomlapackModelRegistry::getInstance();
		$authlevel = $registry->get('authlevel',25); // Default authlevel = Super Administrators only
		$gid = $user->gid;
		switch($authlevel)
		{
			case 25:
				// Super administrator access only

				if($gid != 25)
				{
					return false;
				}
				break;

			case 24:
				// Administrator access allowed
				if( ($gid != 25) && ($gid != 24) )
				{
					return false;
				}
				break;

			case 23:
				// Managers access allowed
				if( ($gid != 25) && ($gid != 24) && ($gid != 23) )
				{
					return false;
				}
				break;
		}

		// All checks passed; you are authenticated
		return true;
	}

	/**
	 * Performs user login so that everything in JUser works
	 */
	function getUserAid( &$user )
	{
		$acl = &JFactory::getACL();

		//Get the user group from the ACL
		$grp = $acl->getAroGroup($user->get('id'));

		// Mark the user as logged in
		$user->set('guest', 0);
		$user->set('aid', 1);

		// Fudge Authors, Editors, Publishers and Super Administrators into the special access group
		if ($acl->is_group_child_of($grp->name, 'Registered')      ||
		$acl->is_group_child_of($grp->name, 'Public Backend')) {
			$user->set('aid', 2);
		}
	}

}