#!/usr/bin/php
<?php
/**
 * @package JoomlaPack
 * @subpackage Utility
 */

// Do not allow being called as a web script
if( array_key_exists('REQUEST_METHOD', $_SERVER) ) die("<h1>Use the command line, Luke!</h1><p>Which means, you should call this script <b>from a command line</b> and not as a web script! You <i>did</i> read the README. Or didn't you?</p>\n");

//define('TEST', 1);
define('_JEXEC', 1);

if(defined('TEST'))
{
	require_once( dirname(__FILE__) . '/../unarchivers/unarchiverparent.php' );
	require_once( dirname(__FILE__) . '/../unarchivers/unjpa.php' );
}

function showHelp()
{
	echo <<<ENDOFHELP
JoomlaPack UnJPA

Usage: /path/to/php unjpa.php archive.jpa

Where:
	/path/to/jpa	The full path to the php executable (php.exe under Windows)
	unjpa.php		The name of this file
	archive.jpa		The name of the JPA archive you'd like to extract
	
Extraction takes place on the working directory. Existing files are replaced.
You have been warned!
ENDOFHELP;
}
// =============================================================================
// Main loop
// =============================================================================

if( count($argv) < 2 )
{
	showHelp();
	exit;
}

$parameters = $argv;

$archiveName = array_shift($parameters);
// When called using PHP, the first argv contains ourselves. If called directly,
// using the shabang sequence on the top of the file to force UNIX systems to
// parse it using PHP, the first argv contains the archive, I think.
if(strpos($archiveName, basename(__FILE__)) !== false)
{
	$archiveName = array_shift($parameters);
}

// Die if no options specified
if( (count($parameters) == 0) && (strpos($archiveName, basename(__FILE__)) !== false) )
{
	showHelp();
	exit;
}

$offset = 0;
$files = 0;
$done = false;

echo "JoomlaPack UnJPA version 2.3\n\n";
echo "Extracting " . $archiveName;

$config = array(
	'filename' => $archiveName
);

$unzip = new CUnJPA($config);

while(!$done)
{
	$ret = $unzip->Extract($offset);

	if($ret === false)
	{
		echo "\n\nAn error occured!\n";
		echo $unzip->getError()."\n\n";
		die();
	}
	echo $ret['file'] . "\n";

	$offset = $ret['offset'];
	$files++;
	$done = $ret['done'];
}

echo "\nFinished\n";
?>
<?php

// Protect from direct execution
defined('_JEXEC') or die('Restricted Access');

/**
 * The parent class of JoomlaPack's archive extraction classes
 * @author Nicholas K. Dionysopoulos
 */
class UnarchiverParent
{
	// Variables set in the configuration
	
	/** @var string The archive's filename */
	var $_filename;
	
	/** @var bool Should we try to restore permissions? */
	var $_flagRestorePermissions = false;
	
	/** @var bool Should we attempt to use FTP mode? If so, we need the FTP class to be set as well */
	var $_flagUseFTP = false;
	
	/** @var string A path to prepend to all stored names */
	var $_addPath = '';
	
	/** @var object An FTP class used for FTP operations */
	var $_ftp = null;
	
	/** @var Array An associative array holding mappings of filenames which should be renamed */
	var $_renameFiles = array();
	
	/** @var Array An indexed array of files which should be skipped during extraction */
	var $_skipFiles = array();
	
	/** @var bool Should I use the JText translation class? */
	var $_flagTranslate = false;
	
	/** @var int Chunk size for uncompressed file copying operations */
	var $_chunkSize = 524288;
	
	// Internal variables
	
	/** @var int The current part number we are reading from */
	var $_currentPart = '';
	
	/** @var bool Did we encounter an error? */
	var $_isError = false;
	
	/** @var string The latest error message */
	var $_error = '';
	
	/** @var Array A hash array of archive parts and their sizes */
	var $_archiveList = array();
	
	/** @var int Total size of archive's parts */
	var $_totalSize = 0;
	
	/** @var resource File pointer to the current part we are reading from */
	var $_fp;
	
	/**
	 * Object constructor for PHP 4 installations
	 * @return UnarchiverAbstract
	 */
	function UnarchiverParent()
	{
		$args = func_get_args();
		call_user_func_array(array(&$this, '__construct'), $args);		
	}
	
	function Extract( $offset = null )
	{
		// Placeholder; implemented in descending classes
		return false;
	}
	
	function getError()
	{
		if(!empty($this->_error))
		{
			return $this->_error;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Public constructor. Takes an associative array of parameters as input.
	 * @param $options An array of options
	 * @return UnarchiverAbstract
	 */
	function __construct( $options = array() )
	{
		if( count($options) > 0 )
		{
			foreach($options as $key => $value)
			{
				switch($key)
				{
					case 'filename': // Archive's absolute filename
						$this->_filename = $value;
						break;
						
					case 'restore_permissions': // Should I restore permissions?
						$this->_flagRestorePermissions = $value;
						break;
						
					case 'use_ftp': // Should I use FTP?
						$this->_flagUseFTP = $value;
						break;
						
					case 'add_path': // Path to prepend
						$this->_addPath = $value;
						break;

					case 'ftp': // The FTP obejct
						$this->_ftp =& $value;
						break;
						
					case 'rename_files': // Which files to rename (hash array)
						$this->_renameFiles = $value;
						break;

					case 'skip_files': // Which files to skip (indexed array)
						$this->_skipFiles = $value;
						break;
						
					case 'translate': // SHould I use the JText translation class?
						$this->_flagTranslate = $value;
						break;
				}				
			}
		}
		
		// Get information about the parts
		$this->_scanArchives();
	}
	
	/**
	 * Returns the base extension of the archive file
	 * @return string The base extension, e.g. '.zip'
	 */
	function _getBaseExtension()
	{
		static $baseextension;
		
		if(empty($baseextension))
		{
			$basename = basename($this->_filename);
			$lastdot = strrpos($basename,'.');
			$baseextension = substr($basename, $lastdot);
		}
		
		return $baseextension;
	}
	
	/**
	 * Scans for multiple parts of an archive set
	 */
	function _scanArchives()
	{
		// Get the components of the archive filename
		$dirname = dirname($this->_filename);
		$base_extension = $this->_getBaseExtension();
		$basename = basename($this->_filename, $base_extension);

		// Scan for multiple parts until we don't find any more of them
		$count = 0;
		$found = true;
		$this->_archiveList = array();
		$totalsize = -1;
		while($found)
		{
			$count++;
			$extension = substr($base_extension, 0, 2).sprintf('%02d', $count);
			$filename = $dirname.DIRECTORY_SEPARATOR.$basename.$extension;
			$found = file_exists($filename);
			if($found)
			{
				// Add yet another part, with a numeric-appended filename
				$rec['name'] = $filename;
				$rec['size'] = @filesize($filename);
			}
			else
			{
				// Add the last part, with the regular extension
				$rec['name'] = $this->_filename;
				$rec['size'] = @filesize($this->_filename);
			}
			$rec['start'] = $totalsize + 1;
			$rec['end'] = $rec['start'] + $rec['size'] - 1;
			$totalsize = $rec['end'];
			$this->_archiveList[$count] = $rec;
		}
		
		$this->_totalSize = $totalsize;
		$this->_currentPart = 1; // Default to start reading from the first part
	}
	
	/**
	 * Makes sure that the _fp variable points to the correct file for a given
	 * offset (relative to the beginning of the first file in a split archive set)
	 * and sets the _currentPart accordingly and skips to the correct offset
	 * @return bool True if we could skip to the offset, false otherwise
	 */
	function _skipToOffset( $offset )
	{
		// Let's find in which archive this offset starts
		$found = false;
		$count = 0;
		while( (!$found) && ($count < count($this->_archiveList)) )
		{
			$count++;
			$found = ($this->_archiveList[$count]['start'] <= $offset) &&
					 ($this->_archiveList[$count]['end'] >= $offset);
		}
		
		// Do we have the correct part set?
		if($this->_currentPart != $count)
		{
			// No, set it and mark that we should open the file pointer
			$this->_currentPart = $count;
			$mustOpen = true;	
		}
		else
		{
			// We are on the correct part. Is it open yet?
			$mustOpen = !is_resource($this->_fp);
		}
		
		// Open the part if we have to
		if($mustOpen)
		{
			if(is_resource($this->_fp)) @fclose($this->_fp);
			$this->_fp = @fopen($this->_archiveList[$this->_currentPart]['name'], 'rb');
			if($this->_fp === false) return false;
		}
		
		// Calculate the relative offset
		$relative_offset = $offset - $this->_archiveList[$this->_currentPart]['start'];
		@fseek($this->_fp, $relative_offset);
		
		return true;
	}
	
	/**
	 * Returns the absolute offset (relative to the start of the first part) or
	 * false if this value is not recoverable
	 * @return int|bool The absolute offset, or false if it failed
	 */
	function _getOffset()
	{
		if(is_resource($this->_fp))
		{
			clearstatcache();
			$relative_offset = @ftell($this->_fp);
			if($relative_offset === false) return false;
			return $relative_offset + $this->_archiveList[$this->_currentPart]['start'];
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Sets the _fp file pointer reference to the next file of a split archive
	 * set and updates the _currentPart accordingly
	 * 
	 * @return bool True if successful, false otherwise
	 */
	function _getNextFile()
	{
		if($this->_currentPart < count($this->_archiveList) )
		{
			return $this->_skipToOffset( $this->_archiveList[$this->_currentPart]['end'] + 1 );
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Returns true if we have reached the end of file
	 * @param $local bool True to return EOF of the local file, false (default) to return if we have reached the end of the archive set
	 * @return bool True if we have reached End Of File
	 */
	function _isEOF($local = false)
	{
		$eof = @feof($this->_fp);
		if($local)
		{
			return $eof;
		}
		else
		{
			return $eof && ($this->_currentPart == count($this->_archiveList) );
		}
	}
	
	/**
	 * Tries to make a directory user-writable so that we can write a file to it
	 * @param $path string A path to a file
	 */
	function _setCorrectPermissions($path)
	{
		$directory = dirname($path);

		if(!$this->_flagUseFTP)
		{
			// Direct file writes mode
			if(!is_dir($directory)) return; // Catch not-a-directory cases
			// Get directories and modify owner permissions to read-write-execute
			$perms = decoct(@fileperms($directory));
			$digit = strlen($perms) == 3 ? 0 : (strlen($perms) == 4 ? 1 : 2);
			$perms = substr_replace($perms, '7', $digit, 1);
			@chmod( $directory, octdec($perms) );
			// Also try to chmod the file itself to 0777 if it exists (in order to allow overwriting)
			@chmod( $path, 0777 );
		}
		else
		{
			// FTP mode
			// As a crappy workaround, I default to 0755 permissions. Oh, well...
			$this->_ftp->chmod($directory, 0755);
			$this->_ftp->chmod($path, 0777);
		}
	}
	
	/**
	 * Tries to recursively create the directory $dirName
	 *
	 * @param string $dirName The directory to create
	 * @return boolean TRUE on success, FALSE on failure
	 * @access private
	 */
	function _createDirRecursive( $dirName )
	{
		$dirArray = explode('/', $dirName);
		$path = '';
		foreach( $dirArray as $dir )
		{
			$path .= $dir . '/';
			$ret = is_dir($path) ? true : @mkdir($path);
			if( !$ret ) {
				$this->_isError = true;
				if($this->_flagTranslate)
				{
					$this->_error = JText::sprintf('COULDNT_CREATE_DIR',$path);
				}
				else
				{
					$this->_error = 'Could not create directory '.$path;
				}
				return false;
			}
			// Try to set new directory permissions to 0755
			@chmod($path, 0755);
		}
		return true;
	}
}
?><?php

class CUnJPA extends UnarchiverParent
{
	/**
	 * Data read from archive's header
	 * @var array
	 */
	var $headerData = array();

	/**
	 * Extracts a file from the JPA archive
	 *
	 * @param integer $offset The offset to start extracting from. If ommited, or set to null,
	 * it continues from the JPA file's current location.
	 * @return array|boolean A return array or FALSE if an error occured
	 */
	function Extract( $offset = null )
	{
		global $ftp;
		
		// This flag is set to true when a "banned" filename is detected
		$isBannedFile = false;
		
		// Generate a return array
		$retArray = array(
			"file"				=> '',		// File name extracted
			"compressed"		=> 0,		// Compressed size
			"uncompressed"		=> 0,		// Uncompressed size
			"type"				=> "file",	// File type (file | dir)
			"compression"		=> "none",	// Compression type (none | gzip | bzip2)
			"offset"			=> 0,			// Offset in JPA file
			"permissions"		=> 0,		// UNIX permissions stored in the archive
			"done"				=> false	// Are we done with extracting files?
		);
		
		$offset = is_null($offset) ? 0 : $offset;
		
		$this->_skipToOffset($offset);
		if($offset == 0) $this->_ReadHeader();
		
		// Get and decode Entity Description Block
		$signature = fread($this->_fp, 3);

		// Check signature
		if( $signature == 'JPF' )
		{
			// This a JPA Entity Block. Process the header.
				
			// Read length of EDB and of the Entity Path Data
			$length_array = unpack('vblocksize/vpathsize', fread($this->_fp, 4));
			// Read the path data
			$file = fread( $this->_fp, $length_array['pathsize'] );
				
			// Handle file renaming
			if(is_array($this->_renameFiles) && (count($this->_renameFiles) > 0) )
			{
				if(array_key_exists($file, $this->_renameFiles))
				{
					$file = $this->_renameFiles[$file];
				}
			}
			
			// Read and parse the known data portion
			$bin_data = fread( $this->_fp, 14 );
			$header_data = unpack('Ctype/Ccompression/Vcompsize/Vuncompsize/Vperms', $bin_data);
			// Read any unknwon data
			$restBytes = $length_array['blocksize'] - (21 + $length_array['pathsize']);
			if( $restBytes > 0 ) $junk = fread($this->_fp, $restBytes);
				
			$compressionType = $header_data['compression'];
				
			// Populate the return array
			$retArray['file'] = $file;
			$retArray['compressed'] = $header_data['compsize'];
			$retArray['uncompressed'] = $header_data['uncompsize'];
			switch($header_data['type'])
			{
				case 0:
					$retArray['type'] = 'dir';
					break;
		
				case 1:
					$retArray['type'] = 'file';
					break;		

				case 2:
					$retArray['type'] = 'link';
					break;		
			}
			switch( $compressionType )
			{
				case 0:
					$retArray['compression'] = 'none';
					break;
				case 1:
					$retArray['compression'] = 'gzip';
					break;
				case 2:
					$retArray['compression'] = 'bzip2';
					break;
			}
			$retArray['permissions'] = $header_data['perms'];
				
			// Find hard-coded banned files
			if( (basename($file) == ".") || (basename($file) == "..") )
			{
				$isBannedFile = true;
			}
			
			// Also try to find banned files passed in class configuration
			if(count($this->_skipFiles) > 0)
			{
				if(in_array($retArray['file'], $this->_skipFiles))
				{
					$isBannedFile = true;
				}
			}
			
			// If we have a banned file, let's skip it
			if($isBannedFile)
			{
				$retArray['offset'] = $this->_getOffset() + $retArray['uncompressed'];
				return $retArray;
			}

			// Last chance to prepend a path to the filename
			if(!empty($this->_addPath))
			{
				$last_addpath_char = substr($this->_addPath, -1);
				if( ($last_addpath_char == '\\') || $last_addpath_char == '/' )
				{
					$this->_addPath = substr($this->_addPath, 0, -1);
				}
				$retArray['file'] = $this->_addPath.'/'.$retArray['file'];
			}
			
			// Do we need to create the directory?
			if(strpos($retArray['file'], '/') > 0) {
				$lastSlash = strrpos($retArray['file'], '/');
				$dirName = substr( $retArray['file'], 0, $lastSlash);
				if(!$this->_flagUseFTP)
				{
					$perms = $this->_flagRestorePermissions ? $retArray['permissions'] : 0755;
					if( $this->_createDirRecursive($dirName, $perms) == false ) {
						$this->_isError = true;
						if($this->_flagTranslate)
						{
							$this->_error = JText::sprintf('COULDNT_CREATE_DIR', $dirName);
						}
						else
						{
							$this->_error = 'Could not create directory '.$dirName;
						}
						return false;
					}
				}
				else
				{
					if( $this->_ftp->makeDirectory($dirName) == false ) {
						$this->_isError = true;
						if($this->_flagTranslate)
						{
							$this->_error = JText::sprintf('COULDNT_CREATE_DIR', $dirName);
						}
						else
						{
							$this->_error = 'Could not create directory '.$dirName;
						}
						return false;
					}
				}
			}

			switch( $retArray['type'] )
			{
				case "dir":
					if(!$this->_flagUseFTP)
					{
						$perms = $this->_flagRestorePermissions ? $retArray['permissions'] : 0755;
						$result = $this->_createDirRecursive($dirName, $perms);
					}
					else
					{
						$perms = $this->_flagRestorePermissions ? $retArray['permissions'] : 0755;
						$result = $this->_ftp->makeDirectory($dirName, $perms);
					}
					if( !$result ) {
						return false;
					}
					break;
						
				case "file":
					switch( $compressionType )
					{
						case 0: // No compression
							if(!$this->_flagUseFTP)
							{
								$outfp = @fopen( $retArray['file'], 'w' );
							}
							else
							{
								$tmpLocal = tempnam(TEMPDIR,'jpks');
								$outfp = @fopen( $tmpLocal, 'w' );
							}
							// Magic permissions handling attempt
							if( ($outfp === false) )
							{
								if(!$this->_flagUseFTP)
								{
									$this->_setCorrectPermissions($retArray['file']);
									$outfp = @fopen( $retArray['file'], 'w' );
								}
								else
								{
									$this->_setCorrectPermissions($tmpLocal);
									$outfp = @fopen( $tmpLocal, 'w' );
								}
							}
							// Re-test
							if( $outfp === false ) {
								// An error occured
								$this->_isError = true;
								if($this->_flagTranslate)
								{
									$this->_error = JText::sprintf('COULDNT_WRITE_FILE', $retArray['file']);
								}
								else
								{
									$this->_error = 'Could not write to file '.$retArray['file'];
								}
								return false;
							}
								
							if( $retArray['uncompressed'] > 0 )
							{
								$readBytes = 0;
								$toReadBytes = 0;
								$leftBytes = $retArray['compressed'];


								while( $leftBytes > 0)
								{
									$toReadBytes = ($leftBytes > $this->_chunkSize) ? $this->_chunkSize : $leftBytes;
									$data = fread( $this->_fp, $toReadBytes );
									$reallyReadBytes = strlen($data);
									$leftBytes -= $reallyReadBytes;
									if($reallyReadBytes < $toReadBytes)
									{
										// We read less than requested! Why? Did we hit local EOF?
										if( $this->_isEOF(true) && !$this->_isEOF(false) )
										{
											// Yeap. Let's go to the next file
											$this->_getNextFile();
										}
										else
										{
											// Nope. The archive is corrupt
											// @todo Translate!
											$this->_error = 'Corrupt archive detected; can\'t continue';
											return false;
										}
									}
								
									fwrite( $outfp, $data );
								}
							}
								
							fclose($outfp);
							if($this->_flagUseFTP)
							{
								$perms = $this->_flagRestorePermissions ? $retArray['permissions'] : 0755;
								$this->_ftp->uploadAndDelete($retArray['file'], $tmpLocal, $perms);
							}
							else
							{
								// Try to change file permissions to 0755
								$perms = $this->_flagRestorePermissions ? $retArray['permissions'] : 0755;
								@chmod($retArray['file'], $perms);
							}
								
							break;
								
						case 1: // GZip compression
						case 2: // BZip compression
							$zipData = fread( $this->_fp, $retArray['compressed'] );
							while( strlen($zipData) < $retArray['compressed'] )
							{
								// End of local file before reading all data, but have more archive parts?
								if($this->_isEOF(true) && !$this->_isEOF(false))
								{
									// Yeap. Read from the next file
									$this->_getNextFile();
									$bytes_left = $retArray['compressed'] - strlen($zipData);
									$zipData .= fread( $this->_fp, $bytes_left );
								}
								else
								{
									// Crap... this archive is corrupt
									// @todo Translate!
									$this->_error = 'Corrupt archive detected; can\'t continue';
									return false;
								}
							}
							if($compressionType == 1)
							{
								$unzipData = gzinflate( $zipData );
							}
							elseif($compressionType == 2)
							{
								$unzipData = bzdecompress( $zipData );
							}
							unset($zipData);

							// Try writing to the output file
							if(!$this->_flagUseFTP)
							{
								$outfp = @fopen( $retArray['file'], 'w' );
							}
							else
							{
								$tmpLocal = tempnam(TEMPDIR,'jpks');
								$outfp = @fopen( $tmpLocal, 'w' );
							}
							// Magic permissions handling attempt
							if( ($outfp === false) )
							{
								if(!$this->_flagUseFTP)
								{
									$this->_setCorrectPermissions($retArray['file']);
									$outfp = @fopen( $retArray['file'], 'w' );
								}
								else
								{
									$this->_setCorrectPermissions($tmpLocal);
									$outfp = @fopen( $tmpLocal, 'w' );
								}
							}
							// Re-test
							if( $outfp === false ) {
								// An error occured
								$this->_isError = true;
								if($this->_flagTranslate)
								{
									$this->_error = JText::sprintf('COULDNT_WRITE_FILE', $retArray['file']);
								}
								else
								{
									$this->_error = 'Could not write to file '.$retArray['file'];
								}
								return false;
							} else {
								// No error occured. Write to the file.
								fwrite( $outfp, $unzipData, $retArray['uncompressed'] );
								fclose( $outfp );
								if($this->_flagUseFTP)
								{
									$perms = $this->_flagRestorePermissions ? $retArray['permissions'] : 0755;
									$ftp->uploadAndDelete($retArray['file'], $tmpLocal, $perms);
								}
								else
								{
									// Try to change file permissions to 0755
									$perms = $this->_flagRestorePermissions ? $retArray['permissions'] : 0755;
									@chmod($retArray['file'], $perms);
								}
							}
							unset($unzipData);
							break;
					}
					break;
					
				case 'link':
					// Symbolic link
					$readBytes = 0;
					$toReadBytes = 0;
					$leftBytes = $retArray['compressed'];
					$data = '';

					while( $leftBytes > 0)
					{
						$toReadBytes = ($leftBytes > $this->_chunkSize) ? $this->_chunkSize : $leftBytes;
						$data .= fread( $this->_fp, $toReadBytes );
						$reallyReadBytes = strlen($data);
						$leftBytes -= $reallyReadBytes;
						if($reallyReadBytes < $toReadBytes)
						{
							// We read less than requested! Why? Did we hit local EOF?
							if( $this->_isEOF(true) && !$this->_isEOF(false) )
							{
								// Yeap. Let's go to the next file
								$this->_getNextFile();
							}
							else
							{
								// Nope. The archive is corrupt
								// @todo Translate!
								$this->_error = 'Corrupt archive detected; can\'t continue';
								return false;
							}
						}
					}
					
					// Try to remove an existing file or directory by the same name
					if(file_exists($retArray['file'])) { @unlink($retArray['file']); @rmdir($retArray['file']); }
					// Remove any trailing slash
					if(substr($retArray['file'], -1) == '/') $retArray['file'] = substr($retArray['file'], 0, -1);
					// Create the symlink
					@symlink($data, $retArray['file']);
					break; 
			}

			$retArray['offset'] = $this->_getOffset();
			return $retArray;
		} else {
			// This is not a file header. This means we are done.
			$retArray['done'] = true;
			return $retArray;
		}
	}
	
	/**
	 * Reads the files header
	 * @access private
	 * @return boolean TRUE on success
	 */
	function _ReadHeader()
	{
		// Initialize header data array
		$this->headerData = array();
		
		// Fail for unreadable files
		if( $this->_fp === false ) return false;

		// Read the signature
		$sig = fread( $this->_fp, 3 );

		if ($sig != 'JPA') return false; // Not a JoomlaPack Archive?

		// Read and parse header length
		$header_length_array = unpack( 'v', fread( $this->_fp, 2 ) );
		$header_length = $header_length_array[1];

		// Read and parse the known portion of header data (14 bytes)
		$bin_data = fread($this->_fp, 14);
		$header_data = unpack('Cmajor/Cminor/Vcount/Vuncsize/Vcsize', $bin_data);

		// Load any remaining header data (forward compatibility)
		$rest_length = $header_length - 19;
		if( $rest_length > 0 )
		$junk = fread($this->_fp, $rest_length);
		else
		$junk = '';

		$this->headerData = array(
			'signature' => 			$sig,
			'length' => 			$header_length,
			'major' => 				$header_data['major'],
			'minor' => 				$header_data['minor'],
			'filecount' => 			$header_data['count'],
			'uncompressedsize' => 	$header_data['uncsize'],
			'compressedsize' => 	$header_data['csize'],
			'unknowndata' => 		$junk
		);

		return true;
	}
	
}
?>